@@ -2,7 +2,7 @@
 import os.path, time, os, sys, re, threading, StringIO, pango, string
 import Image
 import gtk.glade, gtk, gobject, gtk.gdk, traceback
-#import valueEditor, batchEditor
+import batchEditor
 import recipeManager
 import exporters.printer as printer
 import prefs, prefsGui, shopgui, reccard, fnmatch, tempfile
@@ -10,7 +10,7 @@ import exporters, importers
 from exporters.exportManager import get_export_manager
 from importers.importManager import get_import_manager
 import convert, version
-from gtk_extras import ratingWidget, WidgetSaver
+from gtk_extras import ratingWidget, WidgetSaver, mnemonic_manager
 from gtk_extras import dialog_extras as de
 from gtk_extras import treeview_extras as te
 from ImageExtras import get_pixbuf_from_jpg
@@ -23,7 +23,7 @@ from timer import show_timer
 _ = gettext.gettext
 from defaults.defaults import lang as defaults
 import plugin_loader, plugin, plugin_gui
-
+from threadManager import get_thread_manager, get_thread_manager_gui, SuspendableThread
 from zipfile import BadZipfile
 
 if os.name == 'posix':
@@ -104,7 +104,7 @@ class GourmetApplication:
         # These first two items might be better handled using a
         # singleton design pattern... 
         self.splash_label = splash_label
-        self.conv = convert.Converter() 
+        self.conv = convert.get_converter() 
         self.star_generator = ratingWidget.StarGenerator()        
         # Setup methods...
         self.setup_prefs() # Setup preferences...
@@ -140,12 +140,14 @@ class GourmetApplication:
             else:
                 convert.USE_FRACTIONS = convert.FRACTIONS_OFF
         self.prefsGui.apply_prefs_dic['useFractions'] = toggleFractions
-        if self.prefs.get('useFractions',
-                          defaults.LANG_PROPERTIES['useFractions']
-                          ):
-            convert.USE_FRACTIONS = convert.FRACTIONS_NORMAL
-        else:
-            convert.USE_FRACTIONS = convert.FRACTIONS_OFF
+        # Call our method once with the default prefs to apply saved
+        # user settings
+        toggleFractions(None,
+                        self.prefs.get('useFractions',
+                                       defaults.LANG_PROPERTIES['useFractions']
+                                       )
+                        )
+
 
     # Splash convenience method for start-up splashscreen
     def update_splash (self, text):
@@ -197,25 +199,12 @@ class GourmetApplication:
         We load our recipe database from recipeManager. If there's any problem,
         we display the traceback to the user so they can send it out for debugging
         (or possibly make sense of it themselves!)."""
-        try:
-            self.rd = recipeManager.default_rec_manager()
-            check_for_data_to_import(self.rd)
-            # initiate autosave stuff
-            # autosave every 3 minutes (milliseconds * 1000 milliseconds/second * 60 seconds/minute)
-            gobject.timeout_add(1000*60*3,lambda *args: self.rd.save() or True)
-        except:
-            self.prefs['db_backend'] = None
-            self.prefs.save()
-            from StringIO import StringIO
-            f = StringIO()
-            traceback.print_exc(file=f)
-            error_mess = f.getvalue()
-            de.show_message(label='Database Connection failed.',
-                            sublabel='There was a problem with the database information you gave Gourmet',
-                            expander=(_('_Details'),
-                                      error_mess)
-                            )
-            raise
+        self.rd = recipeManager.default_rec_manager()
+        check_for_data_to_import(self.rd)
+        # initiate autosave stuff autosave every 3 minutes
+        # (milliseconds * 1000 milliseconds/second * 60
+        # seconds/minute)
+        gobject.timeout_add(1000*60*3,lambda *args: self.rd.save() or True)
         # connect hooks to modify our view whenever and
         # whenceever our recipes are updated...
         self.rd.modify_hooks.append(self.update_attribute_models)
@@ -352,7 +341,8 @@ class GourmetApplication:
         while len(model) > len(slist):
             last = model.get_iter(len(model) - 1)
             model.remove(last)
-        return model
+
+            return model
 
     def create_attribute_list (self, attribute):
         """Create a ListModel with unique values of attribute.
@@ -472,8 +462,9 @@ class GourmetApplication:
                 c.save_properties()
             if r.edited and de.getBoolean(parent=self.app,
                                              label=_("Save your edits to %s")%r.current_rec.title):
-                r.saveEditsCB()
-            else: r.edited=False # in case someone else checks this (e.g. reccard on close)
+                r.recipe_editor.save_cb()
+            else:
+                r.edited=False # in case someone else checks this (e.g. reccard on close)
         for conf in self.sl.conf:
             conf.save_properties()
         self.prefs.save()
@@ -498,7 +489,7 @@ class GourmetApplication:
                             debug("Unable to terminate thread %s"%t,0)
                             # try not to lose data if this is going to
                             # end up in a force quit
-                            self.save_default() 
+                            #self.save_default() 
                             return True
                 if not use_threads:
                     for t in self._threads:
@@ -508,7 +499,7 @@ class GourmetApplication:
                         except:
                             # try not to lose data if this is going to
                             # end up in a force quit
-                            self.save_default()
+                            #self.save_default()
                             return True
             else:
                 return True
@@ -521,6 +512,20 @@ class GourmetApplication:
         for r in self.rc.values():
             r.hide()
 
+class SuspendableDeletions (SuspendableThread):
+
+    def __init__ (self, recs, name=None):
+        self.recs = recs
+        self.rg = get_application()
+        SuspendableThread.__init__(self, name=name)
+
+    def do_run (self):
+        tot = len(self.recs)
+        for n,r in enumerate(self.recs):
+            self.check_for_sleep()
+            self.rg.rd.delete_rec(r)
+            self.emit('progress',float(n)/tot,_('Permanently deleted %s of %s recipes')%(n,tot))
+
 class RecTrash (RecIndex):
 
     default_searches = [{'column':'deleted','operator':'=','search':True}]
@@ -605,11 +610,6 @@ class RecTrash (RecIndex):
         self.rg.message(_('Undeleted recipes ') + msg)
 
     def purge_selected_recs (self, *args):
-        if not use_threads and self.rg.lock.locked_lock():
-            de.show_message(label=_('An import, export or deletion is running'),
-                            sublabel=_('Please wait until it is finished to delete recipes.')
-                            )
-            return
         debug("recTreeDeleteRec (self, *args):",5)
         sel = self.rectree.get_selection()
         if not sel: return
@@ -732,285 +732,39 @@ class ImporterExporter:
     def print_recs (self, *args):
         debug('printing recipes',3)
         recs = self.get_selected_recs_from_rec_tree()
-        gt.gtk_leave()
-        printer.RecRenderer(self.rd, recs,
-                            dialog_title=gettext.ngettext('Print %s recipe',
-                                                          'Print %s recipes',
-                                                          len(recs))%len(recs),
-                            dialog_parent = self.app,
-                            change_units = self.prefs.get('readableUnits',True)
-                            )
-        gt.gtk_enter()
-
-    def export_selected_recs (self, *args): self.do_export(export_all=False)
-    def export_all_recs (self, *args): self.do_export(export_all=True)
-
-    def do_export (self, export_all=False):
-        if export_all: recs = self.rd.fetch_all(self.rd.recipe_table,deleted=False,sort_by=[('title',1)])
-        else: recs = self.get_selected_recs_from_rec_tree()
-        self.exportManager.offer_multiple_export(recs,
-                                                 self.prefs,
-                                                 prog=self.set_progress_thr,
-                                                 parent=self.app.get_toplevel())
-
-    def import_pre_hook (self, *args):
-        debug('import_pre_hook, gt.gtk_enter()',1)
-        debug('about to run... %s'%self.rd.add_hooks[1:-1],1)
-        gt.gtk_enter()
-
-    def import_post_hook (self, *args):
-        debug('import_post_hook,gt.gtk_leave()',5)
-        gt.gtk_leave()
+        renderer = printer.RecRenderer(self.rd, recs,
+                                       dialog_title=gettext.ngettext('Print %s recipe',
+                                                                     'Print %s recipes',
+                                                                     len(recs))%len(recs),
+                                       dialog_parent = self.app,
+                                       change_units = self.prefs.get('readableUnits',True)
+                                       )
+        #tm = get_thread_manager()
+        #tmg = get_thread_manager_gui()
+        #tm.add_thread(renderer)
+        #tmg.register_thread_with_dialog(_('Print recipes'),renderer)
+        #tmg.show()
 
     def import_webpageg (self, *args):
-        import importers.html_importer
-        sublabel = _('Enter the URL of a recipe archive or recipe website.')
-        url = de.getEntry(label=_('Enter website address.'),
-                          sublabel=sublabel,
-                          entryLabel=_('Enter URL:'),
-                          entryTip=_('Enter the address of a website or recipe archive. The address should begin with http://'),
-                          default_character_width=60,
-                          )
-        if not url: return
-        if url.find('//')<0:
-            url = 'http://'+url
-        self.show_progress_dialog(None,
-                                  progress_dialog_kwargs={'label':_('Importing recipe from %s')%url,
-                                                      'stop':False,
-                                                      'pause':False,})
-        try:
-            i=importers.html_importer.import_url(
-                url,
-                self.rd,
-                progress=self.set_progress_thr,
-                threaded=True)
-            #self.rd,
-            #    url,
-            #    prog=self.set_progress_thr,
-            #    threaded=True)
-            if type(i)==list:
-                impClass,cant_import = self.prepare_import_classes(i)
-                if impClass:
-                    self.run_import(impClass,url,display_errors=False)
-                else:
-                    raise NotImplementedError("Gourmet cannot import")
-            else:
-                self.run_import(i,url,display_errors=False)
-        except NotImplementedError:
-            sublabel=_('Gourmet does not know how to import site %s')%url
-            sublabel += "\n"
-            sublabel += _('Are you sure %(url)s points to a page with a recipe on it?')%locals()
-            de.show_message(label=_('Unable to import'),
-                            sublabel=sublabel,
-                            message_type=gtk.MESSAGE_ERROR)
-            self.hide_progress_dialog()
-        except BadZipfile:
-            de.show_message(label=_('Unable to unzip'),
-                            sublabel=_('Gourmet is unable to unzip the file %s')%url,
-                            message_type=gtk.MESSAGE_ERROR)
-        except IOError:
-            self.hide_progress_dialog()
-            de.show_traceback(label=_('Unable to retrieve URL'),
-                              sublabel=_("""Gourmet was unable to retrieve the site %s. Are you sure your internet connection is working?  If you can retrieve the site with a webbrowser but continue to get this error, please submit a bug report at %s.""")%(url,BUG_URL)
-                              )
-            raise
-        #except gt.Terminated:
-        #    if self.threads > 0: self.threads = self.threads - 1
-        #    self.lock.release()
-        except:
-            self.hide_progress_dialog()
-            de.show_traceback(
-                label=_('Error retrieving %(url)s.')%locals(),
-                sublabel=_('Are you sure %(url)s points to a page with a recipe on it?')%locals()
-                )
-            raise
-        self.make_rec_visible()
+        self.importManager.offer_web_import(parent=self.app.get_toplevel())
 
     def do_import (self, *args):
         self.importManager.offer_import(self.window)
 
-    def importg (self, *args):
-        if not use_threads and self.lock.locked_lock():
-            de.show_message(label=_('An import, export or deletion is running'),
-                            sublabel=_('Please wait until it is finished to start your import.')
-                            )
-            return
-        import_directory = "%s/"%self.prefs.get('rec_import_directory',None)
-        debug('show import dialog',0)
-        ifiles=de.select_file(
-            _("Import Recipes"),
-            filename=import_directory,
-            filters=importers.FILTERS,
-            action=gtk.FILE_CHOOSER_ACTION_OPEN,
-            select_multiple=True)
-        if ifiles:
-            self.prefs['rec_import_directory']=os.path.split(ifiles[0])[0]
-            self.import_multiple_files(ifiles)
-        self.make_rec_visible()
-            
-    def prepare_import_classes (self, files):
-        """Handed multiple import files, prepare to import.
-
-        We return a tuple (importerClasses,cant_import)
-
-        importClass - an instance of importers.importer.MultipleImport
-        which handles the actual import when their run methods are
-        called.
-
-        cant_import - a list of files we couldn't import.
-
-        This does most of the work of import_multiple_files, but
-        leaves it up to our caller to display our progress dialog,
-        etc.
-        """
-        impClass = None            
-        importerClasses = []
-        cant_import = []
-        # we're going to make a copy of the list and chew it up. We do
-        # this rather than doing a for loop because zip files or other
-        # archives can end up expanding our list
-        imp_files = files[0:] 
-        while files:
-            fn = files.pop()
-            if type(fn)==str and os.path.splitext(fn)[1] in ['.gz','.gzip','.zip','.tgz','.tar','.bz2']:
-                try:
-                    debug('trying to unzip %s'%fn,0)
-                    from importers.zip_importer import archive_to_filelist
-                    archive_files = archive_to_filelist(fn)
-                    for a in archive_files:
-                        if type(a) == str:
-                            files += [a]
-                        else:
-                            # if we have file objects, we're going to write
-                            # them out to real files, so we don't have to worry
-                            # about details later (this is stupid, but I'm sick of
-                            # tracking down places where I e.g. closed files regardless
-                            # of whether I opened them)
-                            finame=tempfile.mktemp(a.name)
-                            tfi=open(finame,'w')
-                            tfi.write(a.read())
-                            tfi.close()
-                            files += [finame]
-                    continue
-                except:
-                    cant_import.append(fn)
-                    raise
-                    continue
-            try:
-                impfilt = importers.FILTER_INFO[importers.select_import_filter(fn)]
-            except NotImplementedError:
-                cant_import.append(fn)
-                continue
-            impClass = impfilt['import']({'file':fn,
-                                          'rd':self.rd,
-                                          'threaded':True,
-                                          })
-            if impfilt['get_source']:
-                if type(fn)==str: fname = fn
-                else: fname = "file"
-                source=de.getEntry(label=_("Default source for recipes imported from %s")%fname,
-                                   entryLabel=_('Source:'),
-                                   default=os.path.split(fname)[1], parent=self.app)
-                # the 'get_source' dict is the kwarg that gets
-                # set to the source
-                impClass[2][impfilt['get_source']]=source
-            if impClass: importerClasses.append((impClass,fn))
-            else:
-                debug('GOURMET cannot import file %s'%fn)
-        if importerClasses:
-            impClass = importers.importer.MultipleImporter(self,
-                                                           importerClasses)
-            return impClass,cant_import
+    def do_export (self, export_all=False):
+        if not hasattr(self,'exportManager'):
+            self.exportManager = get_export_manager()
+        if export_all:
+            recs = self.rd.fetch_all(self.rd.recipe_table,deleted=False,sort_by=[('title',1)])
         else:
-            return None,cant_import
+            recs = self.get_selected_recs_from_rec_tree()
+        self.exportManager.offer_multiple_export(
+            recs,
+            self.prefs,
+            prog=self.set_progress_thr,
+            parent=self.app.get_toplevel())
 
-    def import_multiple_files (self, files):
-        """Import multiple files,  showing dialog."""
-        # This should probably be moved to importer with a quiet/not quiet options
-        # and all of the necessary connections handed as arguments.
-        impClass,cant_import=self.prepare_import_classes(files)
-        if impClass:
-            filenames = filter(lambda x: isinstance(x,str), files)
-            self.run_import(
-                impClass,
-                import_source=string.join([os.path.split(f)[1] for f in filenames],", ")
-                )
-        if cant_import:
-            # if this is a file with a name...
-            BUG_URL="http://sourceforge.net/tracker/?group_id=108118&atid=649652"
-            sublabel = gettext.ngettext("Gourmet could not import the file %s",
-                                            "Gourmet could not import the following files: %s",
-                                            len(cant_import))%", ".join(cant_import)
-            sublabel += "\n"
-            sublabel += gettext.ngettext(
-                "If you believe this file is in one of the formats Gourmet supports, please submit a bug report at %s and attach the file.",
-                "If you believe these files are in a format Gourmet supports, please submit a bug report at %s and attach the file.",
-                len(cant_import))%BUG_URL
-            self.offer_url(
-                label=gettext.ngettext("Cannot import file.",
-                                       "Cannot import files.",
-                                       len(cant_import)),
-                sublabel=sublabel,
-                url=BUG_URL)
-            return
 
-    def run_import (self, impClass, import_source="", display_errors=True):
-        """Run our actual import and display progress dialog."""
-        # we have to make sure we don't filter while we go (to avoid
-        # slowing down the process too much).
-        self.wait_to_filter=True
-        self.last_impClass = impClass
-        pre_hooks = [lambda *args: self.inginfo.disconnect_manually()]
-        post_hooks = [lambda *args: self.inginfo.reconnect_manually()]
-        pre_hooks.append(lambda *args: self.rd.add_hooks.insert(0,self.import_pre_hook))
-        pre_hooks.append(lambda *args: self.rd.add_hooks.append(self.import_post_hook))
-        self.threads += 1
-        release = lambda *args: self.lock.release()
-        post_hooks.extend([self.import_cleanup,
-                           lambda *args: setattr(self,'last_impClass',None),
-                           release])
-        def show_progress_dialog (t):
-            debug('showing progress dialog',3)
-            gt.gtk_enter()
-            if import_source:
-                sublab = _('Importing recipes from %s')%import_source
-            else: sublab = None
-            self.rg.show_progress_dialog(
-                t,
-                {'label':_('Importing Recipes'),
-                 'sublabel':sublab
-                 })
-            gt.gtk_leave()
-        pre_hooks.insert(0,show_progress_dialog)
-        pre_hooks.insert(0, lambda *args: self.lock.acquire())
-        t=gt.SuspendableThread(impClass,name="import",
-                               pre_hooks=pre_hooks, post_hooks=post_hooks,
-                               display_errors=display_errors)
-        if self.lock.locked_lock():
-            de.show_message(label=_('An import, export or deletion is running'),
-                            sublabel=_('Your import will start once the other process is finished.'))
-        debug('starting thread',2)
-        debug('PRE_HOOKS=%s'%t.pre_hooks,1)
-        debug('POST_HOOKS=%s'%t.post_hooks,1)
-        t.start()
-
-    @plugin_loader.pluggable_method
-    def import_cleanup (self, *args):
-        """Remove our threading hooks"""
-        debug('import_cleanup!',1)
-        self.rd.add_hooks.remove(self.import_pre_hook)
-        self.rd.add_hooks.remove(self.import_post_hook)
-        debug('hooks: %s'%self.rd.add_hooks,1)
-        self.wait_to_filter=False
-        gt.gtk_enter()
-        # Update our models for category, cuisine, etc.
-        self.update_attribute_models()
-        # Reset our index view
-
-        self.redo_search()
-        gt.gtk_leave()
-
-    
 class StuffThatShouldBePlugins:
     # As you can tell by the name, everything in this class should
     # really be re-implemented as a plugin. Once that process is
@@ -1041,7 +795,6 @@ class StuffThatShouldBePlugins:
             self.sl.addRec(r,mult,d)
             self.sl.show()
 
-
     def batch_edit_recs (self, *args):
         recs = self.get_selected_recs_from_rec_tree()
         if not hasattr(self,'batchEditor'):
@@ -1076,25 +829,12 @@ class StuffThatShouldBePlugins:
                             self.rd.modify_rec(r,changes)
                     else:
                         self.rd.modify_rec(r,self.batchEditor.values)
-                    self.update_rec_iter(r)
+                    self.rmodel.update_recipe(r)
             else:
                 print 'Cancelled'
         self.batchEditor.dialog.hide()
         self.update_attribute_models()
 
-    def showValueEditor (self, *args):
-        if not hasattr(self,'ve'):
-            self.ve=valueEditor.ValueEditor(self.rd,self)
-        self.ve.valueDialog.connect('response',lambda d,r: (r==gtk.RESPONSE_APPLY and self.update_attribute_models))
-        self.ve.show()
-
-    #def email_recs (self, *args):
-    #    debug('email_recs called!',1)
-    #    recs = self.get_selected_recs_from_rec_tree()
-    #    d=recipe_emailer.EmailerDialog(recs, self.rd, self.prefs, self.conv)
-    #    d.setup_dialog()
-    #    d.email()
-
 ui = '''<ui>
 <menubar name="RecipeIndexMenuBar">
   <menu name="File" action="File">
@@ -1128,6 +868,7 @@ ui = '''<ui>
   <menu name="Tools" action="Tools">
     <placeholder name="StandaloneTool">
     <menuitem action="Timer"/>
+    <menuitem action="FieldEditor"/>
     </placeholder>
     <separator/>
     <placeholder name="DataTool"/>
@@ -1159,7 +900,14 @@ ui = '''<ui>
 '''
 
 class RecGui (RecIndex, GourmetApplication, ImporterExporter, StuffThatShouldBePlugins, plugin_loader.Pluggable):
+
+    __single = None
+    
     def __init__ (self, splash_label=None):
+        if RecGui.__single:
+            raise RecGui.__single
+        else:
+            RecGui.__single = self
         self.doing_multiple_deletions = False
         GourmetApplication.__init__(self, splash_label=splash_label)
         self.setup_index_columns()
@@ -1172,12 +920,16 @@ class RecGui (RecIndex, GourmetApplication, ImporterExporter, StuffThatShouldBeP
                           rg=self,
                           editable=False)
         self.setup_database_hooks()        
-        self.setup_main_window()
         self.ui_manager.insert_action_group(self.search_actions,0)
+        self.setup_main_window()
         self.window.add_accel_group(self.ui_manager.get_accel_group())
         self.setup_column_display_preferences()
         plugin_loader.Pluggable.__init__(self,
                                          [plugin.MainPlugin,plugin.ToolPlugin])
+        self.mm = mnemonic_manager.MnemonicManager()
+        self.mm.add_toplevel_widget(self.window)
+        self.mm.fix_conflicts_peacefully()
+        self.window.show()
 
     def setup_hacks (self):
         # THese are properties that we need to set to test with our
@@ -1200,12 +952,10 @@ class RecGui (RecIndex, GourmetApplication, ImporterExporter, StuffThatShouldBeP
                                      self.configure_columns)        
 
     def configure_columns (self, retcolumns):
-        print 'Configure columns->',retcolumns
         hidden=[]
         for c,v in retcolumns:
             if not v: hidden.append(c)
         self.rectree_conf.hidden=self.prefs['rectree_hidden_columns']=hidden
-        print 'Apply visibility'
         self.rectree_conf.apply_visibility()
 
 
@@ -1222,7 +972,7 @@ class RecGui (RecIndex, GourmetApplication, ImporterExporter, StuffThatShouldBeP
             lambda self,*args: (self.doing_multiple_deletions==False and self.redo_search())
             )
         self.rd.modify_hooks.append(self.rmodel.update_recipe)
-        
+
     def selection_changed (self, selected=False):
         if selected != self.selected:
             if selected: self.selected=True
@@ -1233,6 +983,7 @@ class RecGui (RecIndex, GourmetApplication, ImporterExporter, StuffThatShouldBeP
 
     def setup_main_window (self):
         self.window = self.app = gtk.Window()
+        self.window.set_icon_from_file(os.path.join(imagedir,'recbox.ico'))
         self.conf.append(WidgetSaver.WindowSaver(self.window,
                                                  self.prefs.get('app_window',
                                                                 {'window_size':(800,600)}),
@@ -1243,13 +994,18 @@ class RecGui (RecIndex, GourmetApplication, ImporterExporter, StuffThatShouldBeP
         self.window.add(self.main)
         self.window.connect('delete-event',self.quit)
         mb = self.ui_manager.get_widget('/RecipeIndexMenuBar'); mb.show()
-        self.main.pack_start(mb,fill=False,expand=False);        
+        self.main.pack_start(mb,fill=False,expand=False);
+        self.messagebox = gtk.HBox()
+        self.main.pack_start(self.messagebox,fill=False,expand=False)
+        self.main_notebook = gtk.Notebook()
         self.recipe_index_interface = self.glade.get_widget('recipeIndexBox')
         self.recipe_index_interface.unparent()
-        self.main.add(self.recipe_index_interface)
+        self.main_notebook.append_page(self.recipe_index_interface,
+                                       tab_label=gtk.Label(_('Search recipes')))
+        self.main.add(self.main_notebook)
         self.recipe_index_interface.show()
+        self.main_notebook.show(); self.main_notebook.set_show_tabs(False)
         self.main.show()
-        self.window.show()
 
     def setup_actions (self):
         self.ui_manager = gtk.UIManager()
@@ -1262,11 +1018,12 @@ class RecGui (RecIndex, GourmetApplication, ImporterExporter, StuffThatShouldBeP
             ('DeleteRec',gtk.STOCK_DELETE,_('Delete recipe'),
              'Delete',_('Delete selected recipes'),self.rec_tree_delete_rec_cb),
             ('ExportSelected',None,_('E_xport selected recipes'),
-             None,_('Export selected recipes to file'),self.export_selected_recs),
+             None,_('Export selected recipes to file'),
+             lambda *args: self.do_export(export_all=False)),
             ('Print',gtk.STOCK_PRINT,_('_Print'),
              '<Control>P',None,self.print_recs),
             #('Email', None, _('E-_mail recipes'),
-            # None,None,self.email_recs),
+            #None,None,self.email_recs),
             ('BatchEdit',None,_('Batch _edit recipes'),
              '<Control>E',None,self.batch_edit_recs),
             ('ShopRec','add-to-shopping-list',None,None,None,self.shop_recs)
@@ -1289,7 +1046,7 @@ class RecGui (RecIndex, GourmetApplication, ImporterExporter, StuffThatShouldBeP
             ('ImportWeb',None,_('Import _webpage'),
              None,_('Import recipe from webpage'),self.import_webpageg),
             ('ExportAll',None,_('Export _all recipes'),
-             None,_('Export all recipes to file'),self.export_all_recs),
+             None,_('Export all recipes to file'),lambda *args: self.do_export(export_all=True)),
             ('Plugins',None,_('_Plugins'),
              None,_('Manage plugins which add extra functionality to Gourmet.'),
              lambda *args: plugin_gui.show_plugin_chooser()),
@@ -1310,6 +1067,10 @@ class RecGui (RecIndex, GourmetApplication, ImporterExporter, StuffThatShouldBeP
             ('Tools',None,_('_Tools')),
             ('Timer',None,_('_Timer'),
              None,_('Show timer'),lambda *args: show_timer()),
+            ('FieldEditor',None,_('_Field Editor'),
+             None,_('Edit values for any field across multiple recipes at a time'),
+             lambda *args: self.showValueEditor()
+             ),
             #('UnitConverter',None,_('_Unit Converter'),
             # None,_('Calculate unit conversions'),self.showConverter),
             ])
@@ -1361,11 +1122,24 @@ class RecGui (RecIndex, GourmetApplication, ImporterExporter, StuffThatShouldBeP
         
 
     # Extra callbacks for actions on our treeview
+    @plugin_loader.pluggable_method
+    def get_selected_recs_from_rec_tree (self):
+        return RecIndex.get_selected_recs_from_rec_tree(self)
+
+    @plugin_loader.pluggable_method
+    def update_recipe (self, recipe):
+        return RecIndex.update_recipe(self, recipe)
+
+    @plugin_loader.pluggable_method
+    def redo_search (self, *args):
+        return RecIndex.redo_search(self, *args)
+
     def rec_tree_select_rec (self, *args):
         debug("rec_tree_select_rec (self, *args):",5)
         for rec in self.get_selected_recs_from_rec_tree():
             self.open_rec_card(rec)
     
+
     # Deletion
     def show_deleted_recs (self, *args):
         if not hasattr(self,'recTrash'):
@@ -1373,18 +1147,14 @@ class RecGui (RecIndex, GourmetApplication, ImporterExporter, StuffThatShouldBeP
             self.recTrash.show()
         else:
             self.recTrash.show()
-    
+
+
     def rec_tree_delete_rec_cb (self, *args):
         """Make a watch show up (this can be slow
         if lots of recs are selected!"""
-        #gtk.app.window.set_cursor(gtk.gdk.Cursor(gtk.gdk.WATCH))
-        #gobject.idle_add(self.recTreeDeleteRec)
-        # this seems broken
-        sel=self.rectree.get_selection()
-        if not sel: return
-        mod,rr=sel.get_selected_rows()
-        recs = [mod[path][0] for path in rr]
-        self.rec_tree_delete_recs(recs)
+        self.rec_tree_delete_recs(
+            self.get_selected_recs_from_rec_tree()
+            )
 
     def delete_open_card_carefully (self, rec):
         """Delete any open card windows, confirming if the card is edited.
@@ -1392,9 +1162,9 @@ class RecGui (RecIndex, GourmetApplication, ImporterExporter, StuffThatShouldBeP
         We return True if the user cancels deletion.
         """
         if self.rc.has_key(rec.id):
-            rc = self.rc[rec.id]            
+            rc = self.rc[rec.id] 
             if rc.edited:
-                rc.widget.present()
+                rc.show_edit()
                 if not de.getBoolean(
                     label=_('Delete %s?'),
                     sublabel=_('You have unsaved changes to %s. Are you sure you want to delete?'),
@@ -1402,9 +1172,10 @@ class RecGui (RecIndex, GourmetApplication, ImporterExporter, StuffThatShouldBeP
                     custom_no=gtk.STOCK_CANCEL,
                     cancel=False):
                     return True
-            rc.widget.hide()
+            rc.hide()
             self.del_rc(rec.id)
             
+    @plugin_loader.pluggable_method
     def rec_tree_delete_recs (self, recs):
         cancelled = []
         for rec in recs:
@@ -1417,17 +1188,17 @@ class RecGui (RecIndex, GourmetApplication, ImporterExporter, StuffThatShouldBeP
             self.history,
             make_visible=lambda *args: self.redo_search()
             )
+        self.setup_delete_messagebox(
+            gettext.ngettext('You just moved %s recipe to the trash. You can recover this recipe or permanently delete it at any time by clicking Tools->Open Trash.',
+                             'You just moved %s recipes to the trash. You can recover these recipes or permanently delete them at any time by clicking Tools->Open Trash',
+                             len(recs))%len(recs)
+            )
         self.set_reccount()
         if hasattr(self,'recTrash'):
             self.recTrash.update_from_db()
         self.message(_("Deleted") + ' ' + string.join([(r.title or _('Untitled')) for r in recs],', '))
 
     def purge_rec_tree (self, recs, paths=None, model=None):
-        if not use_threads and self.lock.locked_lock():
-            de.show_message(label=_('An import, export or deletion is running'),
-                            sublabel=_('Please wait until it is finished to delete recipes.')
-                            )
-            return
         if not recs:
             # Do nothing if there are no recipes to delete.
             return
@@ -1446,41 +1217,14 @@ class RecGui (RecIndex, GourmetApplication, ImporterExporter, StuffThatShouldBeP
             tree = te.QuickTree([r.title for r in recs])
             expander = [_("See recipes"),tree]
         if de.getBoolean(parent=self.app,label=bigmsg,sublabel=msg,expander=expander):
-            self.redo_search() # update all
 
-            def show_progress (t):
-                gt.gtk_enter()
-                self.show_progress_dialog(t,
-                                          progress_dialog_kwargs={'label':'Deleting recipes'},
-                                          message=_('Deletion paused'), stop_message=_("Stop deletion"))
-                gt.gtk_leave()
-            def save_delete_hooks (t):
-                self.saved_delete_hooks = self.rd.delete_hooks[0:]
-                self.rd.delete_hooks = []
-            def restore_delete_hooks (t):
-                self.rd.delete_hooks = self.saved_delete_hooks
-            pre_hooks = [
-                lambda *args: self.lock.acquire(),
-                save_delete_hooks,
-                show_progress,
-                ]
-            post_hooks = [
-                restore_delete_hooks,
-                lambda *args: self.lock.release()]
-            t=gt.SuspendableThread(gt.SuspendableDeletions(self, recs),
-                                name='delete',
-                                pre_hooks = pre_hooks,
-                                post_hooks = post_hooks)
-            if self.lock.locked_lock():
-                de.show_message(label=_('An import, export or deletion is running'),
-                                sublabel=_('The recipes will be deleted once the other process is finished.')
-                                )
-            debug('PRE_HOOKS=%s'%t.pre_hooks,1)
-            debug('POST_HOOKS=%s'%t.post_hooks,1)
-            debug('rd.add_hooks=%s'%self.rd.add_hooks,1)
-            gt.gtk_leave()
-            t.start()
-            gt.gtk_enter()
+            deleterThread = SuspendableDeletions(recs,name='delete_recs')
+            deleterThread.connect('done',lambda *args: self.recTrash.update_from_db())
+            tm = get_thread_manager()
+            tmg = get_thread_manager_gui()
+            tm.add_thread(deleterThread)
+            tmg.register_thread_with_dialog(_('Delete Recipes'),deleterThread)
+            tmg.show()
         else:
             return True
 
@@ -1513,6 +1257,27 @@ class RecGui (RecIndex, GourmetApplication, ImporterExporter, StuffThatShouldBeP
             debug("%s %s does not have an ID!"%(rec,rec.title),2)
         debug("returning None",2)
         return None
+
+    # Code to show message/undo-button on deletion
+    def setup_delete_messagebox (self, msg):
+        # Clear existing messages...
+        for child in self.messagebox.get_children():
+            self.messagebox.remove(child)
+        # Add new message
+        l = gtk.Label()
+        close_button = gtk.Button(stock=gtk.STOCK_CLOSE)
+        close_button.connect('clicked',lambda *args: self.messagebox.hide())
+        self.messagebox.pack_end(close_button,expand=False,fill=False)
+        undo_button = gtk.Button(stock=gtk.STOCK_UNDO)
+        show_trash_button = gtk.Button('See Trash Now')
+        show_trash_button.connect('clicked',lambda *args: self.show_deleted_recs() or self.messagebox.hide())
+        undo_button.connect('clicked',lambda *args: self.history[-1].inverse() or self.messagebox.hide())
+        self.messagebox.pack_end(undo_button,expand=False,fill=False)
+        self.messagebox.pack_end(show_trash_button,expand=False,fill=False)
+        self.messagebox.pack_start(l,fill=True,expand=True)
+        self.messagebox.show_all()
+        l.set_markup('<i><span background="yellow">'+msg+'</span></i>')
+        l.set_line_wrap(True)
     # end deletion
 
     # end Extra Callbacks for actions on treeview
@@ -1612,7 +1377,7 @@ def get_application ():
     except RecGui, rg:
         return rg
 
-if __name__ == '__main__' and False:
+if __name__ == '__main__':
     if os.name!='nt':
         import profile, tempfile,os.path
         import hotshot, hotshot.stats
@@ -1629,6 +1394,6 @@ if __name__ == '__main__' and False:
     else:
         startGUI()
 
-elif __name__ == '__main__':
-    rgn = RecGui()
-    gtk.main()
+#elif __name__ == '__main__':
+#    rgn = RecGui()
+#    gtk.main()
