@@ -18,9 +18,9 @@ parser = optparse.OptionParser(
                          dest="printer",default="gnomeprint",
                          help="Force us to use the LPR-based printer even if gnomeprint is available.",
                          ),
-    #optparse.make_option("--choose-database",const="True",action='store_const',
-    #                     dest="choosedb",help="Show Database chooser dialog",
-    #                     default=""),
+    optparse.make_option("--database-url",action='store',type='string',
+                         dest="db_url",help="Custom url for database of form driver://args/location",
+                         default=""),
     optparse.make_option("--plugin-directory",action="store",type="string",
                          dest="html_plugin_dir",help="Directory for webpage import filter plugins.",
                          default="",
@@ -34,14 +34,6 @@ parser = optparse.OptionParser(
                          action='store_const',const=False,
                          dest='threads',
                          help='Disable threading support.'),
-    optparse.make_option("--sqlite",
-                         action="store_const",const="sqlite",
-                         dest="db",help="Use SQLITE database backend",
-                         default="sqlite"),
-    #optparse.make_option("--metakit",
-    #                     action="store_const",const="metakit",
-    #                     dest="db",help="Use metakit backend",
-    #                     default="sqlite"),
     optparse.make_option("--gourmet-directory",
                          action="store",
                          dest="gourmetdir",
