@@ -1,8 +1,8 @@
-# For testing - DELETE ME
-import sys
-sys.path.append('/usr/share/')
-# End for testing - DELETE ME
-
+## For testing - DELETE ME
+#import sys
+#sys.path.append('/usr/share/')
+## End for testing - DELETE ME
+import shutil
 import os.path
 from gourmet.gdebug import debug, TimeAction
 import re, pickle, string, os.path, string, time
@@ -18,8 +18,9 @@ from gourmet.plugin_loader import Pluggable, pluggable_method
 from gourmet.plugin import DatabasePlugin
 
 import sqlalchemy, sqlalchemy.orm
-from sqlalchemy import Integer, Binary, String, Float, Boolean, Numeric, Table, Column, ForeignKey
-from sqlalchemy.sql import and_, or_
+from sqlalchemy import Integer, Binary, String, Float, Boolean, Numeric, Table, Column, ForeignKey, Text
+from sqlalchemy.sql import and_, or_, case 
+from sqlalchemy import func
 
 def map_type_to_sqlalchemy (typ):
     """A convenience method -- take a string type and map it into a
@@ -30,7 +31,7 @@ def map_type_to_sqlalchemy (typ):
         return String(
             length=int(typ[typ.find('(')+1:typ.find(')')])
             )
-    if typ=='text': return String(length=None)
+    if typ=='text': return Text()
     if typ=='bool': return Boolean()
     if typ=='float': return Float()
     if typ=='binary': return Binary()
@@ -59,8 +60,10 @@ def make_simple_select_arg (criteria,*tables):
     for k,v in fix_colnames(criteria,*tables).items():
         if type(v)==tuple:
             operator,value = v
+            if type(v)==str:
+                v = unicode(v)
             if operator=='in':
-                args.append(k.in_(*value))
+                args.append(k.in_(value))
             elif hasattr(k,operator):
                 args.append(getattr(k,operator)(value))
             elif hasattr(k,operator+'_'): # for keywords like 'in'
@@ -78,7 +81,7 @@ def make_simple_select_arg (criteria,*tables):
 
 def make_order_by (sort_by, table, count_by=None, join_tables=[]):
     ret = []
-    for col,dir in sort_by:
+    for col,direction in sort_by:        
         if col=='count' and not hasattr(table.c,'count'):
             col = sqlalchemy.func.count(getattr(table.c,count_by))
         else:
@@ -93,7 +96,12 @@ def make_order_by (sort_by, table, count_by=None, join_tables=[]):
                         break
                 if broken:
                     raise ValueError("No such column for tables %s %s: %s"%(table, join_tables, col))
-        if dir==1: # Ascending
+        if isinstance(col.type, Text):
+            # Sort nulls last rather than first using case statement...
+            col = case([(col == None, 'z'*20),
+                        (col == '', 'z'*20),
+                        ],else_=func.lower(col))
+        if direction==1: # Ascending
             ret.append(sqlalchemy.asc(col))
         else:
             ret.append(sqlalchemy.desc(col))
@@ -101,8 +109,6 @@ def make_order_by (sort_by, table, count_by=None, join_tables=[]):
     
 class DBObject:
     pass
-
-
 # CHANGES SINCE PREVIOUS VERSIONS...
 # categories_table: id -> recipe_id, category_entry_id -> id
 # ingredients_table: ingredient_id -> id, id -> recipe_id
@@ -145,8 +151,9 @@ class RecData (Pluggable):
         self.add_ing_hooks = []
         timer = TimeAction('initialize_connection + setup_tables',2)
         self.initialize_connection()
-        Pluggable.__init__(self,[DatabasePlugin])        
+        Pluggable.__init__(self,[DatabasePlugin])            
         self.setup_tables()
+        self.metadata.create_all()
         self.update_version_info(gourmet.version.version)
         self._created = True
         timer.end()
@@ -157,6 +164,7 @@ class RecData (Pluggable):
         """Initialize our database connection.
         
         This should also set self.new_db accordingly"""
+        debug('Initializing DB connection',1)
         if self.filename:
             self.new_db = not os.path.exists(self.filename)
             print 'Connecting to file ',self.filename
@@ -165,6 +173,8 @@ class RecData (Pluggable):
         self.db = sqlalchemy.create_engine(self.url,strategy='threadlocal') 
         self.db.begin()
         self.metadata = sqlalchemy.MetaData(self.db)
+        # Be noisy... (uncomment for debugging/fiddling)
+        # self.metadata.bind.echo = True
         try:
             self.session = sqlalchemy.orm.create_session()
         except AttributeError:
@@ -181,9 +191,11 @@ class RecData (Pluggable):
         if self.url.startswith('sqlite'):
             sqlite_connection = self.db.connect().connection
             sqlite_connection.create_function('regexp',2,regexp)
-            c = sqlite_connection.cursor()
-            c.execute('select name from sqlite_master')
+            #c = sqlite_connection.cursor()
+            #c.execute('select name from sqlite_master')
             #sqlite_connection.create_function('instr',2,instr)
+        self.db.commit() # Somehow necessary to prevent "DB Locked" errors 
+        debug('Done initializing DB connection',1)
 
     def save (self):
         """Save our database (if we have a separate 'save' concept)"""
@@ -204,13 +216,12 @@ class RecData (Pluggable):
 
     def _setup_object_for_table (self, table, klass):
         self.__table_to_object__[table] = klass
-        print 'Mapping ',repr(klass),'->',repr(table)
+        #print 'Mapping ',repr(klass),'->',repr(table)
         if True in [col.primary_key for col in table.columns]:
             sqlalchemy.orm.mapper(klass,table)
         else:
             # if there's no primary key...
-            sqlalchemy.orm.mapper(klass,table,
-                                  primary_key=[table.oid_column])
+            raise "All tables need a primary key -- specify 'rowid'/Integer/Primary Key in table spec for %s"%table
 
     @pluggable_method
     def setup_tables (self):
@@ -221,7 +232,6 @@ class RecData (Pluggable):
         self.__table_to_object__ = {}
         self.setup_base_tables()
         self.setup_shopper_tables() # could one day be part of a plugin
-        self.metadata.create_all()
 
     def setup_base_tables (self):
         self.setup_info_table()
@@ -234,22 +244,39 @@ class RecData (Pluggable):
                                 Column('version_super',Integer(),**{}), # three part version numbers 2.1.10, etc. 1.0.0
                                 Column('version_major',Integer(),**{}),
                                 Column('version_minor',Integer(),**{}),
-                                Column('last_access',Integer(),**{})
-                                 )
+                                Column('last_access',Integer(),**{}),
+                                Column('rowid',Integer(),**{'primary_key':True})
+                                )
         class Info (object):
             pass
         self._setup_object_for_table(self.info_table, Info)
+        self.plugin_info_table = Table('plugin_info',self.metadata,
+                                       Column('plugin',Text(),**{}),
+                                       # three part version numbers
+                                       # 2.1.10, etc. 1.0.0 -- these
+                                       # contain the Gourmet version
+                                       # at the last time of
+                                       # plugging-in
+                                       Column('id',Integer(),**{'primary_key':True}),
+                                       Column('version_super',Integer(),**{}), 
+                                       Column('version_major',Integer(),**{}),
+                                       Column('version_minor',Integer(),**{}),
+                                       # Stores the last time the plugin was used...
+                                       Column('plugin_version',String(length=32),**{}))
+        class PluginInfo (object):
+            pass
+        self._setup_object_for_table(self.plugin_info_table, PluginInfo)
 
     def setup_recipe_table (self):
         self.recipe_table = Table('recipe',self.metadata,
                                   Column('id',Integer(),**{'primary_key':True}),
-                                  Column('title',String(length=None),**{}),
-                                  Column('instructions',String(length=None),**{}),
-                                  Column('modifications',String(length=None),**{}),
-                                  Column('cuisine',String(length=None),**{}),
+                                  Column('title',Text(),**{}),
+                                  Column('instructions',Text(),**{}),
+                                  Column('modifications',Text(),**{}),
+                                  Column('cuisine',Text(),**{}),
                                   Column('rating',Integer(),**{}),
-                                  Column('description',String(length=None),**{}),
-                                  Column('source',String(length=None),**{}),
+                                  Column('description',Text(),**{}),
+                                  Column('source',Text(),**{}),
                                   Column('preptime',Integer(),**{}),
                                   Column('cooktime',Integer(),**{}),
                                   Column('servings',Float(),**{}),
@@ -260,7 +287,7 @@ class RecData (Pluggable):
                                   Column('recipe_hash',String(length=32),**{}),
                                   # A hash for uniquely identifying a recipe (based on ingredients)
                                   Column('ingredient_hash',String(length=32),**{}),
-                                  Column('link',String(length=None),**{}), # A field for a URL -- we ought to know about URLs
+                                  Column('link',Text(),**{}), # A field for a URL -- we ought to know about URLs
                                   Column('last_modified',Integer(),**{}),
                                   ) # RECIPE_TABLE_DESC
         class Recipe (object): pass
@@ -270,7 +297,7 @@ class RecData (Pluggable):
         self.categories_table = Table('categories',self.metadata,
                                     Column('id',Integer(),primary_key=True),
                                     Column('recipe_id',Integer,ForeignKey('recipe.id'),**{}), #recipe ID
-                                    Column('category',String(length=None),**{}) # Category ID
+                                    Column('category',Text(),**{}) # Category ID
                                     ) # CATEGORY_TABLE_DESC
         class Category (object): pass
         self._setup_object_for_table(self.categories_table,Category)
@@ -280,15 +307,15 @@ class RecData (Pluggable):
                                        Column('id',Integer(),primary_key=True),
                                        Column('recipe_id',Integer,ForeignKey('recipe.id'),**{}),
                                        Column('refid',Integer,ForeignKey('recipe.id'),**{}),
-                                       Column('unit',String(length=None),**{}),
+                                       Column('unit',Text(),**{}),
                                        Column('amount',Float(),**{}),
                                        Column('rangeamount',Float(),**{}),
-                                       Column('item',String(length=None),**{}),
-                                       Column('ingkey',String(length=None),**{}),
+                                       Column('item',Text(),**{}),
+                                       Column('ingkey',Text(),**{}),
                                        Column('optional',Boolean(),**{}),
                                        #Integer so we can distinguish unset from False
                                        Column('shopoptional',Integer(),**{}), 
-                                       Column('inggroup',String(length=None),**{}),
+                                       Column('inggroup',Text(),**{}),
                                        Column('position',Integer(),**{}),
                                        Column('deleted',Boolean(),**{}),
                                        )
@@ -299,9 +326,9 @@ class RecData (Pluggable):
         # Keylookup table - for speedy keylookup
         self.keylookup_table = Table('keylookup',self.metadata,
                                      Column('id',Integer(),primary_key=True),
-                                     Column('word',String(length=None),**{}),
-                                      Column('item',String(length=None),**{}),
-                                      Column('ingkey',String(length=None),**{}),
+                                     Column('word',Text(),**{}),
+                                      Column('item',Text(),**{}),
+                                      Column('ingkey',Text(),**{}),
                                       Column('count',Integer(),**{})
                                      ) # INGKEY_LOOKUP_TABLE_DESC
         class KeyLookup (object): pass
@@ -313,8 +340,8 @@ class RecData (Pluggable):
 
         # shopcats - Keep track of which shoppin category ingredients are in...
         self.shopcats_table = Table('shopcats',self.metadata,
-                                    Column('ingkey',String(length=None),**{'primary_key':True}),
-                                    Column('shopcategory',String(length=None),**{}),
+                                    Column('ingkey',Text(),**{'primary_key':True}),
+                                    Column('shopcategory',Text(),**{}),
                                     Column('position',Integer(),**{}),
                                     )
         class ShopCat (object): pass
@@ -322,7 +349,7 @@ class RecData (Pluggable):
         
         # shopcatsorder - Keep track of the order of shopping categories
         self.shopcatsorder_table = Table('shopcatsorder',self.metadata,
-                                         Column('shopcategory',String(length=None),**{'primary_key':True}),
+                                         Column('shopcategory',Text(),**{'primary_key':True}),
                                          Column('position',Integer(),**{}),
                                          )
         class ShopCatOrder (object): pass
@@ -331,7 +358,7 @@ class RecData (Pluggable):
         # pantry table -- which items are in the "pantry" (i.e. not to
         # be added to the shopping list)
         self.pantry_table = Table('pantry',self.metadata,
-                                  Column('ingkey',String(length=None),**{'primary_key':True}),
+                                  Column('ingkey',Text(),**{'primary_key':True}),
                                   Column('pantry',Boolean(),**{}),
                                   )
         class Pantry (object): pass
@@ -368,12 +395,35 @@ class RecData (Pluggable):
             pass
         self._setup_object_for_table(self.convtable_table, Convtable)
 
+    def backup_db (self):
+        """Make a backup copy of the DB -- this ensures experimental
+        code won't permanently screw our users."""
+        import time, os.path
+        backup_file_name = self.filename + '.backup-' + time.strftime('%d-%m-%y')
+        while os.path.exists(backup_file_name):
+            backup_file_name += 'I'
+        print 'Making a backup copy of DB in ',backup_file_name
+        print 'You can use it to restore if something ugly happens.'
+        shutil.copy(self.filename,backup_file_name) # Make a backup...
+        import gourmet.gtk_extras.dialog_extras as de
+        import gtk
+        de.show_message(
+            title=_("Upgrading database"),
+            label=_("Upgrading database"),            
+            sublabel=_("Depending on the size of your database, this may be an intensive process and may take  some time. Your data has been automatically backed up in case something goes wrong."),
+            expander=(_("Details"),_("A backup has been made in %s in case something goes wrong. If this upgrade fails, you can manually rename your backup file recipes.db to recover it for use with older Gourmet.")%backup_file_name),
+            message_type=gtk.MESSAGE_INFO)
+
     def update_version_info (self, version_string):
         """Report our version to the database.
 
         If necessary, we'll do some version-dependent updates to the GUI
         """
         stored_info = self.fetch_one(self.info_table)
+        version = [s for s in version_string.split('.')]
+        current_super = int(version[0])
+        current_major = int(version[1])
+        current_minor = int(version[2])
         if not stored_info or not stored_info.version_major:
             # Default info -- the last version before we added the
             # version tracker...
@@ -381,25 +431,29 @@ class RecData (Pluggable):
                              'version_major':11,
                              'version_minor':0}
             if not stored_info:
-                self.do_add(self.info_table,
-                            default_info)
+                if not self.new_db:
+                    self.do_add(self.info_table,
+                                default_info)
+                else:
+                    self.do_add(self.info_table,
+                                {'version_super':current_super,
+                                 'version_major':current_major,
+                                 'version_minor':current_minor,}
+                                )
             else:
                 self.do_modify(
                     self.info_table,
                     stored_info,
                     default_info)
             stored_info = self.fetch_one(self.info_table)            
-        version = [s for s in version_string.split('.')]
-        current_super = int(version[0])
-        current_major = int(version[1])
-        current_minor = int(version[2])
-        ### Code for updates between versions...
         
-        if not self.new_db:
+        ### Code for updates between versions...
+        if not self.new_db:  
             # Version < 0.11.4 -> version >= 0.11.4... fix up screwed up keylookup_table tables...
             # We don't actually do this yet... (FIXME)
             #print 'STORED_INFO:',stored_info.version_super,stored_info.version_major,stored_info.version_minor
             if stored_info.version_super == 0 and stored_info.version_major <= 11 and stored_info.version_minor <= 3:
+                self.backup_db()
                 print 'Fixing broken ingredient-key view from earlier versions.'
                 # Drop keylookup_table table, which wasn't being properly kept up
                 # to date...
@@ -410,6 +464,7 @@ class RecData (Pluggable):
                     self.add_ing_to_keydic(ingredient.item,ingredient.ingkey)
 
             if stored_info.version_super == 0 and stored_info.version_major < 14:
+                self.backup_db()
                 # Name changes to make working with IDs make more sense
                 # (i.e. the column named 'id' should always be a unique
                 # identifier for a given table -- it should not be used to
@@ -429,15 +484,16 @@ class RecData (Pluggable):
             # Add recipe_hash, ingredient_hash and link fields
             # (These all get added in 0.13.0)
             if stored_info.version_super == 0 and stored_info.version_major <= 12:
+                self.backup_db()                
                 print 'UPDATE FROM < 0.13.0...'
                 # Don't change the table defs here without changing them
                 # above as well (for new users) - sorry for the stupid
                 # repetition of code.
                 self.add_column_to_table(self.recipe_table,('last_modified',Integer(),{}))
-                self.add_column_to_table(self.recipe_table,('recipe_hash','String(length=32)',{}))
-                self.add_column_to_table(self.recipe_table,('ingredient_hash','String(length=32)',{}))
+                self.add_column_to_table(self.recipe_table,('recipe_hash',String(length=32),{}))
+                self.add_column_to_table(self.recipe_table,('ingredient_hash',String(length=32),{}))
                 # Add a link field...
-                self.add_column_to_table(self.recipe_table,('link',String(length=None),{}))
+                self.add_column_to_table(self.recipe_table,('link',Text(),{}))
                 print 'Searching for links in old recipe fields...'
                 URL_SOURCES = ['instructions','source','modifications']
                 recs = self.search_recipes(
@@ -483,11 +539,10 @@ class RecData (Pluggable):
                                 )
                 # Add hash values to identify all recipes...
                 for r in self.fetch_all(self.recipe_table): self.update_hashes(r)
-        for plugin in self.plugins:
-            plugin.update_version(
-                (stored_info.version_super,stored_info.version_major,stored_info.version_minor),
-                (current_super,current_major,current_minor)
-                )
+            for plugin in self.plugins:
+                self.update_plugin_version(plugin,
+                                           (current_super,current_major,current_minor)
+                                           )
         ### End of code for updates between versions...
         if (current_super!=stored_info.version_super
             or
@@ -504,6 +559,51 @@ class RecData (Pluggable):
                 id_col=None
                 )
 
+    def update_plugin_version (self, plugin, current_version=None):
+        if current_version:
+            current_super,current_major,current_minor = current_version
+        else:
+            i = self.fetch_one(self.info_table)
+            current_super,current_major,current_minor = (i.version_super,
+                                                         i.version_major,
+                                                         i.version_minor)
+        existing = self.fetch_one(self.plugin_info_table,
+                                  plugin=plugin.name)
+        if existing:
+            sup,maj,minor,plugin_version = (existing.version_super,
+                                            existing.version_major,
+                                            existing.version_minor,
+                                            existing.plugin_version)
+        else:
+            # Default to the version before our plugin system existed
+            sup,maj,minor = 0,13,9
+            plugin_version = 0
+        try:
+            plugin.update_version(
+                gourmet_stored=(sup,maj,minor),
+                plugin_stored = plugin_version,
+                gourmet_current=(current_super,current_major,current_minor),
+                plugin_current = plugin.version,
+                )
+        except:
+            print 'Problem updating plugin',plugin,plugin.name
+            raise
+        # Now we store the information so we know we've done an update
+        info = {
+            'plugin':plugin.name,
+            'version_super':current_super,
+            'version_major':current_major,
+            'version_minor':current_minor,
+            'plugin_version':plugin.version}
+        if existing and (
+            current_minor != minor or
+            current_major != maj or
+            current_super != sup or
+            plugin.version != plugin_version):
+            self.do_modify(self.plugin_info_table,existing,info)
+        else:
+            self.do_add(self.plugin_info_table,info)
+
     def run_hooks (self, hooks, *args):
         """A basic hook-running function. We use hooks to allow parts of the application
         to tag onto data-modifying events and e.g. update the display"""
@@ -525,18 +625,22 @@ class RecData (Pluggable):
     def fetch_count (self, table, column, sort_by=[],**criteria):
         """Return a counted view of the table, with the count stored in the property 'count'"""
         result =  sqlalchemy.select(
-            [sqlalchemy.func.count(getattr(table.c,column)),
+            [sqlalchemy.func.count(getattr(table.c,column)).label('count'),
              getattr(table.c,column)],
-            group_by=column,
-            order_by=make_order_by(sort_by,table,count_by=column)
+            *make_simple_select_arg(criteria,table),
+            **{'group_by':column,
+               'order_by':make_order_by(sort_by,table,count_by=column),
+               }
             ).execute().fetchall()
-        for row in result: row.count = row[0]
         return result
 
     def fetch_len (self, table, **criteria):
         """Return the number of rows in table that match criteria
         """
-        return table.count().execute().fetchone()[0]
+        if criteria:
+            return table.count(*make_simple_select_arg(criteria,table)).execute().fetchone()[0]
+        else:
+            return table.count().execute().fetchone()[0]
 
     def fetch_join (self, table1, table2, col1, col2,
                     column_names=None, sort_by=[], **criteria):
@@ -632,6 +736,10 @@ class RecData (Pluggable):
 
         sort_by is a list of tuples (column,1) [ASCENDING] or (column,-1) [DESCENDING]
         """
+        if 'rating' in [t[0] for t in sort_by]:
+            i = [t[0] for t in sort_by].index('rating')
+            d = (sort_by[i][1]==1 and -1 or 1)
+            sort_by[i] = ('rating',d)
         criteria = self.get_criteria((searches,'and'))
         if 'category' in [s[0] for s in sort_by]:
             return sqlalchemy.select([c for c in self.recipe_table.c],# + [self.categories_table.c.category],
@@ -667,7 +775,32 @@ class RecData (Pluggable):
     def get_ingkeys_with_count (self, search={}):
         """Get unique list of ingredient keys and counts for number of times they appear in the database.
         """
-        raise NotImplementedError
+        if search:
+            col = getattr(self.ingredients_table.c,search['column'])
+            operator = search.get('operator','LIKE')
+            if operator=='LIKE':
+                criteria = col.like(search['search'])
+            elif operator=='REGEXP':
+                criteria = col.op('REGEXP')(search['search'])
+            else:
+                criteria = col==crit['search']
+        else:
+            criteria = []
+        result =  sqlalchemy.select(
+            [sqlalchemy.func.count(self.ingredients_table.c.ingkey).label('count'),
+             self.ingredients_table.c.ingkey],
+            criteria,
+            **{'group_by':'ingkey',
+               'order_by':make_order_by([],self.ingredients_table,count_by='ingkey'),
+               }
+            ).execute().fetchall()
+        return result
+        return self.fetch_count(self.ingredients_table,'ingkey',)
+        s = sqlalchemy.select([
+            self.ingredients_table.c.ingkey,
+            func.count(self.ingredients_table.c.ingkey).label('count')
+            ]).execute()
+        return s.fetchall()
 
     def delete_by_criteria (self, table, criteria):
         """Table is our table.
@@ -705,8 +838,13 @@ class RecData (Pluggable):
         """table is a table, column_spec is a tuple defining the
         column, following the format for new tables.
         """
-        #column = Column(*column_spec)
-        raise NotImplementedError
+        name = table.name; new_col = column_spec[0]; coltyp = column_spec[1]
+        sql = 'ALTER TABLE %(name)s ADD %(new_col)s %(coltyp)s;'%locals()
+        try:
+            self.db.execute(sql)
+        except:
+            print 'Ignoring error in add_column_to_table'
+            import traceback; traceback.print_exc()
 
     def alter_table (self, table_name, setup_function, cols_to_change={}, cols_to_keep=[]):
         """Change table, moving some columns.
@@ -725,6 +863,7 @@ class RecData (Pluggable):
         will allow us to e.g. change/add primary key columns to sqlite
         tables
         """
+        print 'Attempting to alter ',table_name,setup_function,cols_to_change,cols_to_keep
         try:
             self.db.execute('ALTER TABLE %(t)s RENAME TO %(t)s_temp'%{'t':table_name})
         except:
@@ -751,7 +890,11 @@ class RecData (Pluggable):
                 newdic[cols_to_change[k]]=getattr(row,k)
             for c in cols_to_keep:
                 newdic[c] = getattr(row,c)
-            self.do_add(getattr(self,'%s_table'%table_name),newdic)
+            try:
+                self.do_add(getattr(self,'%s_table'%table_name),newdic)
+            except:
+                print 'Trouble inserting newdic,',newdic
+                raise
         self.db.execute('DROP TABLE %s_temp'%table_name)
 
     # Metakit has no AUTOINCREMENT, so it has to do special magic here
@@ -969,6 +1112,7 @@ class RecData (Pluggable):
 
     def add_rec (self, dic, accept_ids=False):
         """Dictionary is a dictionary of column values for our recipe.
+        Return the ID of the newly created recipe.
 
         If accept_ids is True, we accept recipes with IDs already
         set. These IDs need to have been reserved with the new_id()
@@ -986,11 +1130,12 @@ class RecData (Pluggable):
             ret = self.do_add_rec(dic)
         except:
             print 'Problem adding recipe with dictionary...'
-            for k,v in dic.items(): print 'KEY:',k,'VALUE:',v
+            for k,v in dic.items(): print 'KEY:',k,'of type',type(k),'VALUE:',v,'of type',type(v)
             raise
         else:
             if type(ret)==int:
                 ID = ret
+                ret = self.get_rec(ID) 
             else:
                 ID = ret.id
             for c in cats:
@@ -1011,12 +1156,44 @@ class RecData (Pluggable):
             print 'Problem adding',dic
             raise
 
+    def add_ings (self, dics):
+        """Add multiple ingredient dictionaries at a time."""
+        for d in dics: self.validate_ingdic(d)
+        try:
+            self.ingredients_table.insert().execute(*dics)
+        except ValueError:
+            for d in dics: self.coerce_types(self.ingredients_table,d)
+            self.ingredients_table.insert().execute(*dics)
+
     # Lower level DB access functions -- hopefully subclasses can
     # stick to implementing these    
 
+    def coerce_types (self, table, dic):
+        """Modify dic to make sure types are correct for table.
+        """
+        type_to_pytype = {Float:float,
+                          Integer:int,
+                          String:str,
+                          Boolean:bool,
+                          Numeric:float,
+                          }
+        for k,v in dic.copy().items():
+            column_obj = getattr(table.c,k)
+            if column_obj.type.__class__ in type_to_pytype:
+                try:
+                    v = type_to_pytype[column_obj.type.__class__](v)
+                except:
+                    v = None
+                if dic[k] != v:
+                    dic[k] = v
+
     def do_add (self, table, dic):
         insert_statement = table.insert()
-        result_proxy = insert_statement.execute(**dic)
+        try:
+            result_proxy = insert_statement.execute(**dic)
+        except ValueError:
+            self.coerce_types(table,dic)
+            result_proxy = insert_statement.execute(**dic)
         return result_proxy
 
     def do_add_and_return_item (self, table, dic, id_prop='id'):
@@ -1240,7 +1417,9 @@ class RecData (Pluggable):
         else:
             return amt
 
-    def get_amount_and_unit (self, ing, mult=1, conv=None,fractions=convert.FRACTIONS_ALL):
+    @pluggable_method
+    def get_amount_and_unit (self, ing, mult=1, conv=None, fractions=None, adjust_units=False,
+                             favor_current_unit=True,preferred_unit_groups=[]):
         """Return a tuple of strings representing our amount and unit.
         
         If we are handed a converter interface, we will adjust the
@@ -1250,18 +1429,20 @@ class RecData (Pluggable):
         unit=ing.unit
         ramount = None
         if type(amt)==tuple: amt,ramount = amt
-        if conv:
-            amt,unit = conv.adjust_unit(amt,unit)
+        if adjust_units or preferred_unit_groups:
+            amt,unit = conv.adjust_unit(amt,unit,
+                                        favor_current_unit=favor_current_unit,
+                                        preferred_unit_groups=preferred_unit_groups)
             if ramount and unit != ing.unit:
                 # if we're changing units... convert the upper range too
                 ramount = ramount * conv.converter(ing.unit, unit)
         if ramount: amt = (amt,ramount)
-        return (self._format_amount_string_from_amount(amt,fractions=fractions),unit)
+        return (self._format_amount_string_from_amount(amt,fractions=fractions,unit=unit),unit)
         
     def get_amount_as_string (self,
                               ing,
                               mult=1,
-                              fractions=convert.FRACTIONS_ALL
+                              fractions=None,
                               ):
         """Return a string representing our amount.
         If we have a multiplier, multiply the amount before returning it.        
@@ -1269,18 +1450,29 @@ class RecData (Pluggable):
         amt = self.get_amount(ing,mult)
         return self._format_amount_string_from_amount(amt, fractions=fractions)
 
-    def _format_amount_string_from_amount (self, amt, fractions=convert.FRACTIONS_ALL):
+    def _format_amount_string_from_amount (self, amt, fractions=None, unit=None):
         """Format our amount string given an amount tuple.
 
+        If fractions is None, we use the default setting from
+        convert.USE_FRACTIONS. Otherwise, we will override that
+        setting.
+        
         If you're thinking of using this function from outside, you
         should probably just use a convenience function like
         get_amount_as_string or get_amount_and_unit
         """
+        if fractions is None:
+            # None means use the default value
+            fractions = convert.USE_FRACTIONS
+        if unit:
+            approx = defaults.unit_rounding_guide.get(unit,0.01)
+        else:
+            approx = 0.01
         if type(amt)==tuple:
-            return "%s-%s"%(convert.float_to_frac(amt[0],fractions=fractions).strip(),
-                            convert.float_to_frac(amt[1],fractions=fractions).strip())
+            return "%s-%s"%(convert.float_to_frac(amt[0],fractions=fractions,approx=approx).strip(),
+                            convert.float_to_frac(amt[1],fractions=fractions,approx=approx).strip())
         elif type(amt) in (float,int):
-            return convert.float_to_frac(amt,fractions=fractions)
+            return convert.float_to_frac(amt,fractions=fractions,approx=approx)
         else: return ""
 
     def get_amount_as_float (self, ing, mode=1): #1 == self.AMT_MODE_AVERAGE
@@ -1452,7 +1644,8 @@ class RecipeManager (RecData):
     def __init__ (self,*args,**kwargs):
         debug('recipeManager.__init__()',3)
         RecData.__init__(self,*args,**kwargs)
-        self.km = keymanager.KeyManager(rm=self)
+        #self.km = keymanager.KeyManager(rm=self)
+        self.km = keymanager.get_keymanager(rm=self)
         
     def key_search (self, ing):
         """Handed a string, we search for keys that could match
