@@ -579,6 +579,10 @@ CONVERTER_TABLE = {
     ("qt", "c"):4,
     ("gallon", "qt"):4,
     ("l", "qt"):1.057,
+    ('Japanese cup','ml'):200,
+    ('metric cup','ml'):250,
+    ('Imperial cup','ml'):284.130625,
+    ('Imperial pint','oz'):20,
     ("l", "ml"):1000,
     ("l", "cl"):100,
     ("l", "dl"):10,
