@@ -21,7 +21,8 @@ class exporter (SuspendableThread, Pluggable):
     DEFAULT_ENCODING = 'utf-8'
 
     name='exporter'
-
+    ALLOW_PLUGINS_TO_WRITE_NEW_FIELDS = True
+    
     def __init__ (self, rd, r, out,
                   conv=None,
                   imgcount=1,
@@ -60,7 +61,7 @@ class exporter (SuspendableThread, Pluggable):
         self.fractions=fractions
         self.use_ml=use_ml
         self.convert_attnames = convert_attnames
-        if not conv: conv=convert.Converter()
+        if not conv: conv=convert.get_converter()
         self.conv=conv
         self.imgcount=imgcount
         self.images = []
@@ -511,6 +512,7 @@ class ExporterMultirec (SuspendableThread, Pluggable):
         
     @pluggable_method
     def do_run (self):
+        print 'Exportermultirec.do_run'
         self.rcount = 0
         self.rlen = len(self.recipe_table)        
         if not self.one_file:
