@@ -41,14 +41,12 @@ class GladeCustomHandlers:
 
         def custom_handler (glade,func_name,
                             widg, s1,s2,i1,i2):
-            print 'Glade wants ',func_name
             f=getattr(self,func_name)
             w= f(s1,s2,i1,i2)
             return w
         gtk.glade.set_custom_handler(custom_handler)
 
     def add_custom_handler (self, handler_name, handler):
-        print 'register custom handler',handler_name,handler
         if hasattr(self,handler_name):
             import traceback; traceback.print_stack()
             print 'WARNING: ','We already had a handler named %s'%handler
@@ -322,8 +320,11 @@ def launch_url (url, ext=""):
         os.startfile(url)
     elif os.name == 'posix':
         try:
-            import gnome
-            gnome.url_show(url)
+            if hasattr(gtk,'show_uri'):
+                gtk.show_uri(gtk.gdk.Screen(),url,0L)
+            else:
+                import gnome
+                gnome.url_show(url)
         except ImportError:
             print 'gnome libraries not available, trying builtins'
             if not ext: ext=os.path.splitext(url)
@@ -357,11 +358,8 @@ empty_model = gtk.ListStore(str)
 
 # Set up custom STOCK items and ICONS!
 icon_factory = gtk.IconFactory()
-for filename,stock_id,label,modifier,keyval in [
-    ('Nutrition.png','nutritional-info',_('Nutritional Information'),0,0),
-    ('AddToShoppingList.png','add-to-shopping-list',_('Add to _Shopping List'),gtk.gdk.CONTROL_MASK,gtk.gdk.keyval_from_name('l')),
-    ('recbox.png','recipe-box',None,0,0),
-    ]:
+    
+def add_icon (file_name, icon_name, label=None, modifier=0, keyval=0):
     pb = gtk.gdk.pixbuf_new_from_file(os.path.join(imagedir,filename))
     iconset = gtk.IconSet(pb)
     icon_factory.add(stock_id,iconset)
@@ -371,3 +369,11 @@ for filename,stock_id,label,modifier,keyval in [
                     modifier,
                     keyval,
                     "")])
+
+for filename,stock_id,label,modifier,keyval in [    
+    ('AddToShoppingList.png','add-to-shopping-list',_('Add to _Shopping List'),gtk.gdk.CONTROL_MASK,gtk.gdk.keyval_from_name('l')),
+    ('recbox.png','recipe-box',None,0,0),
+    ('reccard.png','recipe-card',None,0,0),
+    ('reccard_edit.png','edit-recipe-card',None,0,0),    
+    ]:
+    add_icon(filename,stock_id,label,modifier,keyval)
