@@ -1,4 +1,5 @@
 ### Copyright (C) 2005 Thomas M. Hinkle
+### Copyright (C) 2009 Rolf Leggewie
 ###
 ### This library is free software; you can redistribute it and/or
 ### modify it under the terms of the GNU General Public License as
@@ -12,8 +13,8 @@
 ###
 ### You should have received a copy of the GNU General Public License
 ### along with this library; if not, write to the Free Software
-### Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
-### 02111-1307, USA.
+### Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
+### USA 
 
 import pango,gtk, xml.sax.saxutils
 from gourmet.gdebug import *
@@ -75,7 +76,7 @@ class PangoBuffer (gtk.TextBuffer):
         try:
             self.parsed,self.txt,self.separator = pango.parse_markup(txt,u'\x00')
         except:
-            print 'Escaping text, we seem to have a problem here!'
+            print 'Problem encountered escaping text: "%s"'%txt
             import traceback; traceback.print_exc()
             txt=xml.sax.saxutils.escape(txt)
             self.parsed,self.txt,self.separator = pango.parse_markup(txt,u'\x00')
