@@ -1,4 +1,4 @@
-import gtk, gtk.gdk, gobject
+import gtk, gtk.gdk, gobject, pango
 from gourmet.ImageExtras import get_pixbuf_from_jpg
 from gourmet.gtk_extras.thumbnail import check_for_thumbnail,fetched_uris
 from gourmet.gtk_extras.dialog_extras import ModalDialog
@@ -26,10 +26,15 @@ def grab_thumbnail (uri, type, iqueue, pqueue, progress_portion=1, progress_star
     pqueue.append(('Getting %s'%uri,0))
     import time
     #print 'Fetching ',uri
-    fi = check_for_thumbnail(uri,type,reporthook)
+    try:
+        fi = check_for_thumbnail(uri,type,reporthook)
+    except:
+        print 'WARNING: Error on creating thumbnail - ignoring'
+        import traceback; traceback.print_exc()
+    else:
+        iqueue.append((fi,uri))
     #print 'Fetched'
     #print 'Adding result to Queue'
-    iqueue.append((fi,uri))
     #print 'Done'
     
 class ImageBrowser (gtk.IconView):
@@ -146,7 +151,7 @@ class ImageBrowserDialog (ModalDialog):
         self.ib.connect('selection-changed',self.selection_changed_cb)
         self.ib.connect('item-activated',self.okcb)
         self.sw = gtk.ScrolledWindow()
-        self.pb = gtk.ProgressBar()
+        self.pb = gtk.ProgressBar(); self.pb.set_ellipsize(pango.ELLIPSIZE_MIDDLE)
         self.vbox.pack_end(self.sw)
         self.vbox.pack_end(self.pb,expand=False)
         self.ib.progressbar = self.pb
