@@ -1,14 +1,11 @@
 import gtk, gtk.glade, gobject, re, os, os.path
 import gglobals, convert
-import WidgetSaver
-import cb_extras as cb
-import dialog_extras as de
+from gtk_extras import WidgetSaver, mnemonic_manager, pageable_store
+from gtk_extras import cb_extras as cb
+from gtk_extras import dialog_extras as de
 from gettext import gettext as _
 from gettext import ngettext
-import mnemonic_manager
-import pageable_store
-import hildon
-
+import nutrition.nutritionDruid as nutritionDruid
 
 class KeyEditor:
 
@@ -20,7 +17,7 @@ class KeyEditor:
         self.glade = gtk.glade.XML(os.path.join(gglobals.gladebase,'keyeditor.glade'))        
         self.rd = rd
         self.rg = rg
-        self.widget_names = ['treeview', 'searchByBox', 'searchEntry', 'searchButton',
+        self.widget_names = ['treeview', 'searchByBox', 'searchEntry', 'searchButton', 'window',
                              'searchAsYouTypeToggle', 'regexpTog',
                              'changeKeyEntry',
                              'changeItemEntry',
@@ -35,18 +32,6 @@ class KeyEditor:
                         'unit':self.changeUnitEntry,
                         'amount':self.changeAmountEntry,
                         }
-
-        self.window = hildon.Window()
-        self.window.set_default_size(800, 480)
-        self.window.set_title('Key Editor')
-        self.window.connect("key-press-event", self.on_key_press)
-        self.window.connect("window-state-event", self.on_window_state_change)
-        self.window_in_fullscreen = False 
-
-        gtkwindow = self.glade.get_widget('window')
-        alignment15 = self.glade.get_widget('alignment15')
-        alignment15.reparent(self.window)
-
         # setup entry callback to sensitize/desensitize apply
         self.applyEntriesButton.set_sensitive(False)
         self.clearEntriesButton.set_sensitive(False)
@@ -81,6 +66,7 @@ class KeyEditor:
             'applyEntries':self.applyEntriesCB,
             'clearEntries':self.clearEntriesCB,
             'close_window': lambda *args: self.window.hide(),
+            'editNutritionalInfo':self.editNutritionalInfoCB,
             })
         # setup mnemonic manager
         self.mm = mnemonic_manager.MnemonicManager()
@@ -103,23 +89,6 @@ class KeyEditor:
             self.rg.prefs.get('regexpTog',
                            {'active':self.regexpTog.get_active()}),
             ['toggled']))
-
-        gtkwindow.destroy()
-        self.window.show_all()
-
-    def on_window_state_change(self, widget, event, *args):           
-        if event.new_window_state & gtk.gdk.WINDOW_STATE_FULLSCREEN:
-            self.window_in_fullscreen = True
-        else:
-            self.window_in_fullscreen = False
-
-    def on_key_press(self, widget, event, *args):            
-        if event.keyval == gtk.keysyms.F6:
-            if self.window_in_fullscreen:
-                self.window.unfullscreen()
-            else:
-                self.window.fullscreen()
-
                 
     def dont_ask_cb (self, widget, *args):
         self.dont_ask=widget.get_active()
@@ -130,15 +99,37 @@ class KeyEditor:
         self.VALUE_COL = 2
         self.COUNT_COL = 3
         self.REC_COL = 4
+        self.NUT_COL = 5
         for n,head in [[self.FIELD_COL,_('Field')],
                        [self.VALUE_COL,_('Value')],
                        [self.COUNT_COL,_('Count')],
-                       [self.REC_COL, _('Recipes')]]:
-            renderer = gtk.CellRendererText()
+                       [self.REC_COL, _('Recipes')],
+                       [self.NUT_COL, _('Nutritional Info')],
+                       ]:
+            if n == self.NUT_COL:
+                renderer = gtk.CellRendererToggle()
+            else:
+                renderer = gtk.CellRendererText()
+            # If we have gtk > 2.8, set up text-wrapping
+            try:
+                renderer.get_property('wrap-width')
+            except TypeError:
+                pass
+            else:
+                renderer.set_property('wrap-mode',gtk.WRAP_WORD)
+                if n == self.FIELD_COL:
+                    renderer.set_property('wrap-width',60)
+                elif n in [self.VALUE_COL,self.REC_COL]: renderer.set_property('wrap-width',250)
+                else: renderer.set_property('wrap-width',100)
             if n==self.VALUE_COL:
                 renderer.set_property('editable',True)
                 renderer.connect('edited',self.tree_edited,n,head)
-            col = gtk.TreeViewColumn(head, renderer, text=n)
+            if n == self.NUT_COL:
+                col = gtk.TreeViewColumn(head, renderer, active=n, visible=n)
+            else:
+                col = gtk.TreeViewColumn(head, renderer, text=n)
+            if n == self.VALUE_COL:
+                col.set_property('expand',True)
             col.set_resizable(True)
             self.treeview.append_column(col)
 
@@ -156,12 +147,12 @@ class KeyEditor:
                              sublabel=_("You won't be able to undo this action. If there are already ingredients with the key \"%s\", you won't be able to distinguish between those items and the items you are changing now."%text)
                              ):
                 self.rd.update(
-                    self.rd.iview,
+                    self.rd.ingredients_table,
                     curdic,
                     {'ingkey':text}
                     )
                 self.rd.delete_by_criteria(
-                    self.rd.ikview,
+                    self.rd.keylookup_table,
                     {'ingkey':key}
                     )
         elif field=='item':
@@ -169,7 +160,7 @@ class KeyEditor:
                              sublabel=_("You won't be able to undo this action. If there are already ingredients with the item \"%s\", you won't be able to distinguish between those items and the items you are changing now.")%text
                              ):
                 self.rd.update(
-                    self.rd.iview,
+                    self.rd.ingredients_table,
                     curdic,
                     {'item':text}
                     ) 
@@ -184,13 +175,13 @@ class KeyEditor:
                               )
             if val==1:
                 self.rd.update(
-                    self.rd.iview,
+                    self.rd.ingredients_table,
                     {'unit':unit},
                     {'unit':text},
                     )
             elif val==2:
                 self.rd.update(
-                    self.rd.iview,
+                    self.rd.ingredients_table,
                     curdic,
                     {'unit':text}
                     )
@@ -216,7 +207,7 @@ class KeyEditor:
             elif val == 3:
                 cond = curdic
             self.rd.update(
-                self.rd.iview,
+                self.rd.ingredients_table,
                 {'unit':unit,'amount':convert.frac_to_float(amount)},
                 {'unit':unit,'amount':new_amount}
                 )
@@ -231,6 +222,12 @@ class KeyEditor:
         self.treeModel.connect('page-changed',self.model_changed_cb)
         self.treeModel.connect('view-changed',self.model_changed_cb)
         
+    def resetTree (self):
+        self.search_string = 'NO ONE WOULD EVER SEARCH FOR THIS HACKISH STRING'
+        curpage = self.treeModel.page
+        self.doSearch()
+        self.treeModel.set_page(curpage)
+
     def doSearch (self):
         """Do the actual searching."""
         last_search = self.search_string
@@ -371,19 +368,63 @@ class KeyEditor:
             curkey = self.treeModel.get_value(itr,self.VALUE_COL)
             if not already_updated:
                 self.rd.update(
-                    self.rd.iview,
+                    self.rd.ingredients_table,
                     curdic,
                     newdic,
                     )
                 if curdic.has_key('ingkey') and newdic.has_key('ingkey'):
                     self.rd.delete_by_criteria(
-                        self.rd.ikview,
+                        self.rd.keylookup_table,
                         {'ingkey':curdic['ingkey']}
                         )
-            self.update_iter(itr,newdic) # A recursive method that
-                                         # will set values for us and
-                                         # our children as necessary
-            updated_iters.append(itr) 
+        self.resetTree()
+            #self.update_iter(itr,newdic) # A recursive method that
+            #                             # will set values for us and
+            #                             # our children as necessary
+            #updated_iters.append(itr) 
+
+    def editNutritionalInfoCB (self, *args):
+        nid = nutritionDruid.NutritionInfoDruid(self.rg.nd, self.rg.prefs)
+        mod,rows = self.treeview.get_selection().get_selected_rows()
+        keys_to_update = {}
+        for path in rows:
+            itr = mod.get_iter(path)
+            # Climb to the key-level for each selection -- we don't
+            # care about anything else.
+            parent = mod.iter_parent(itr)
+            while parent:
+                itr = parent
+                parent = mod.iter_parent(itr)
+            curkey = mod.get_value(itr,self.VALUE_COL)
+            #if mod.get_value(itr,self.NUT_COL):
+            #    print "We can't yet edit nutritional information..."
+            #else:
+            if True:
+                keys_to_update[curkey]=[]
+                child = mod.iter_children(itr)
+                while child:
+                    grandchild = mod.iter_children(child)
+                    while grandchild:
+                        # Grand children are units...
+                        unit = mod.get_value(grandchild,self.VALUE_COL)
+                        amounts = []
+                        greatgrandchild = mod.iter_children(grandchild)
+                        while greatgrandchild:
+                            amount = mod.get_value(
+                                greatgrandchild,
+                                self.VALUE_COL
+                                )
+                            keys_to_update[curkey].append((convert.frac_to_float(amount),unit))
+                            greatgrandchild = mod.iter_next(greatgrandchild)
+                        grandchild = mod.iter_next(grandchild)
+                    child = mod.iter_next(child)
+                nid.add_ingredients(keys_to_update.items())
+                nid.connect('finish',self.update_nutinfo)
+                nid.show()
+            
+    def update_nutinfo (self, *args):
+        print 'KeyEditor.update_nutinfo'
+        self.treeModel.reset_views()
 
     def update_iter (self, itr, newdic):
         """Update iter and its children based on values in newdic"""
@@ -460,7 +501,7 @@ class KeyStore (pageable_store.PageableTreeStore,pageable_store.PageableViewStor
     UNIT = _('Unit')+':'
     AMOUNT = _('Amount')+':'    
     
-    columns = ['obj','ingkey','item','count','recipe']
+    columns = ['obj','ingkey','item','count','recipe','ndbno']
     def __init__ (self, rd, per_page=15):
         self.rd = rd
         pageable_store.PageableTreeStore.__init__(self,
@@ -469,22 +510,37 @@ class KeyStore (pageable_store.PageableTreeStore,pageable_store.PageableViewStor
                                                    str, # value
                                                    int, # count
                                                    str, # recipe
+                                                   int, # nutritional information equivalent
                                                    ],
                                                   per_page=per_page)
 
     def reset_views (self):
         self.view = self.rd.get_ingkeys_with_count()
-        #self.ikview = self.rd.filter(self.rd.ikview,lambda row: row.item)
-        # Limit iview to ingkeys only, then select the unique values of that, then
+        for n in range(self._get_length_()):
+            parent = (n,)
+            path = parent
+            try:
+                itr = self.get_iter(path)
+            except ValueError:
+                print 'Trouble with path',path
+                return
+            self.emit('row-changed',path,itr)
+            child = self.iter_children(itr)
+            while child:
+                path = self.get_path(child)
+                self.emit('row-changed',path,child)
+                child = self.iter_next(child)
+        #self.keylookup_table = self.rd.filter(self.rd.keylookup_table,lambda row: row.item)
+        # Limit ingredients_table to ingkeys only, then select the unique values of that, then
         # filter ourselves to values that have keys
-        #self.view = self.rd.filter(self.rd.iview.project(self.rd.iview.ingkey).unique(),
+        #self.view = self.rd.filter(self.rd.ingredients_table.project(self.rd.ingredients_table.ingkey).unique(),
         #                           lambda foo: foo.ingkey)
 
     def _setup_parent_ (self, *args, **kwargs):
         self.reset_views()
 
     def limit_on_ingkey (self, txt, search_options={}):
-        self.limit(txt,'ingkey',search_options)
+        self.limit(txt,self.rd.ingredients_table+'.ingkey',search_options)
 
     def limit_on_item (self, txt, search_options={}):
         self.limit(txt,'item',search_options)
@@ -516,6 +572,7 @@ class KeyStore (pageable_store.PageableTreeStore,pageable_store.PageableViewStor
                 # avoidable slowdown (look here if code seems sluggish)
                 row.count,
                 None,
+                row.ndbno or 0,
                 ]
 
     def _get_children_ (self,itr):
@@ -524,56 +581,62 @@ class KeyStore (pageable_store.PageableTreeStore,pageable_store.PageableViewStor
         value = self.get_value(itr,2)
         if field==self.KEY:
             ingkey = value
-            for item in self.rd.get_unique_values('item',self.rd.iview,ingkey=ingkey):
+            for item in self.rd.get_unique_values('item',self.rd.ingredients_table,ingkey=ingkey):
                 ret.append([None,
                             self.ITEM,
                             item,
-                            self.rd.fetch_len(self.rd.iview,ingkey=ingkey,item=item),
-                            self.get_recs(ingkey,item)])
+                            self.rd.fetch_len(self.rd.ingredients_table,ingkey=ingkey,item=item),
+                            self.get_recs(ingkey,item),
+                            0
+                            ])
         elif field==self.ITEM:
             ingkey = self.get_value(self.iter_parent(itr),2)
             item = value
-            for unit in self.rd.get_unique_values('unit',self.rd.iview,ingkey=ingkey,item=item):
+            for unit in self.rd.get_unique_values('unit',self.rd.ingredients_table,ingkey=ingkey,item=item):
                 ret.append([None,
                             self.UNIT,
                             unit,
-                            self.rd.fetch_len(self.rd.iview,ingkey=ingkey,item=item,unit=unit),
-                            None])
+                            self.rd.fetch_len(self.rd.ingredients_table,ingkey=ingkey,item=item,unit=unit),
+                            None,
+                            0])
             if not ret:
                 ret.append([None,
                             self.UNIT,
                             '',
                             self.get_value(self.iter_parent(itr),3),
-                            None])                
+                            None,
+                            0])                
         elif field==self.UNIT:
             item = self.get_value(self.iter_parent(itr),2)
             ingkey = self.get_value(self.iter_parent(
                 self.iter_parent(itr)),2)
             unit = self.get_value(itr,2)
             amounts = []
-            for i in self.rd.fetch_all(self.rd.iview,ingkey=ingkey,item=item,unit=unit):
+            for i in self.rd.fetch_all(self.rd.ingredients_table,ingkey=ingkey,item=item,unit=unit):
                 astring = self.rd.get_amount_as_string(i)
                 if astring in amounts: continue
                 ret.append([None,
                             self.AMOUNT,
                             astring,
                             (i.rangeamount
-                             and self.rd.fetch_len(self.rd.iview,
+                             and self.rd.fetch_len(self.rd.ingredients_table,
                                                    ingkey=ingkey,item=item,
                                                    unit=unit,
                                                    amount=i.amount,rangeamount=i.rangeamount)
-                             or  self.rd.fetch_len(self.rd.iview,
+                             or  self.rd.fetch_len(self.rd.ingredients_table,
                                                    ingkey=ingkey,item=item,
                                                    unit=unit,
                                                    amount=i.amount)),
-                            None])
+                            None,
+                            0])
                 amounts.append(astring)
             if not ret:
                 ret.append([None,
                             self.AMOUNT,
                             '',
                             self.get_value(self.iter_parent(itr),3),
-                            None])
+                            None,
+                            0])
             
         return ret
         #row = row[0]
@@ -586,7 +649,7 @@ class KeyStore (pageable_store.PageableTreeStore,pageable_store.PageableViewStor
 
     def get_recs (self, key, item):
         """Return a string with a list of recipes containing an ingredient with key and item"""
-        recs = [i.id for i in self.rd.fetch_all(self.rd.iview,ingkey=key,item=item)]
+        recs = [i.id for i in self.rd.fetch_all(self.rd.ingredients_table,ingkey=key,item=item)]
         titles = []
         looked_at = []
         for r_id in recs:
@@ -602,6 +665,8 @@ if gtk.pygtk_version[1]<8:
 if __name__ == '__main__':
     import recipeManager
     rm = recipeManager.default_rec_manager()
+    import sys
+    sys.path.append(os.path.realpath('../tests'))
     import testExtras
     rg = testExtras.FakeRecGui(rm)
     ke=KeyEditor(rm,rg)
