@@ -1,5 +1,6 @@
 import string, re, time, sys
-from defaults import lang as defaults
+from defaults.defaults import lang as defaults
+from defaults.defaults import langProperties as langProperties
 from gdebug import *
 
 note_separator_regexp = '(;|\s+-\s+|--)'
@@ -17,15 +18,25 @@ class KeyManager:
     MAX_MATCHES = 10
     word_splitter = re.compile('\W+')
 
+    __single = None
+
     def __init__ (self, kd={}, rm=None):
+        if KeyManager.__single:
+            raise KeyManager.__single
+        else:
+            KeyManager.__single = self
         self.kd = kd
+        if not rm:
+            import recipeManager
+            rm = recipeManager.default_rec_manager()
         self.rm = rm
         self.cooking_verbs=cooking_verbs
         # This needs to be made sane i18n-wise
         self.ignored = defaults.IGNORE
         self.ignored.extend(self.cooking_verbs)
         self.ignored_regexp = re.compile("[,; ]?(" + '|'.join(self.ignored) + ")[,; ]?")
-        if not(self.rm.ikview):
+        #if not(self.rm.keylookup_table): # wrong
+        if self.rm.fetch_len(self.rm.keylookup_table) == 0:
             self.initialize_from_defaults()
         self.initialize_categories()
         
@@ -62,7 +73,7 @@ class KeyManager:
         should be flour, barley"""
         debug("Start initialize_categories",10)
         self.cats = []
-        for k in self.rm.get_unique_values('ingkey',self.rm.iview,deleted=False):
+        for k in self.rm.get_unique_values('ingkey',self.rm.ingredients_table,deleted=False):
             fnd=k.find(',')
             if fnd != -1:
                 self.cats.append(k[0:fnd])
@@ -90,7 +101,7 @@ class KeyManager:
 
     def get_key_fast (self, s):
         try:
-            if s: srch = self.rm.fetch_all(self.rm.ikview,
+            if s: srch = self.rm.fetch_all(self.rm.keylookup_table,
                                            item=s,
                                            sort_by=[('count',1)]
                                            )
@@ -134,10 +145,10 @@ class KeyManager:
         if len(main_txts)==1:
             main_txts.extend(defaults.guess_plurals(txt))
         for t in main_txts:
-            is_key = self.rm.fetch_one(self.rm.iview,ingkey=t)
+            is_key = self.rm.fetch_one(self.rm.ingredients_table,ingkey=t)
             if is_key>=0:
                 retvals[t]=.9
-            exact = self.rm.fetch_all(self.rm.ikview,
+            exact = self.rm.fetch_all(self.rm.keylookup_table,
                                       item=t)
             if exact:
                 for o in exact:
@@ -162,7 +173,7 @@ class KeyManager:
         for w in words:
             if not w:
                 continue
-            srch = self.rm.fetch_all(self.rm.ikview,word=w)
+            srch = self.rm.fetch_all(self.rm.keylookup_table,word=w)
             total_count = sum([m.count for m in srch])
             for m in srch:
                 ik = m.ingkey
@@ -189,11 +200,18 @@ class KeyManager:
         """Generate a generic-looking key from a string."""
         timer = TimeAction('keymanager.generate_key 1',3)
         debug("Start generate_key(self,%s)"%ingr,10)
-        ingr = ingr.strip().lower()
+        ingr = ingr.strip()
+        # language specific here - turn off the strip().lower() for German, 'cos:
+        # i) german Nouns always start with an uppercase Letter.
+        # ii) the function 'lower()' doesn't appear to work correctly with umlauts.
+        if (not langProperties['capitalisedNouns']):
+            # We want to use unicode's lower() method
+            if not isinstance(ingr,unicode):
+                ingr = unicode(ingr.decode('utf8'))
+            ingr = ingr.lower()
         timer.end()
         timer = TimeAction('keymanager.generate_key 2',3)
         debug("verbless string=%s"%ingr,10)
-        # we remove final s
         if ingr.find(',') == -1:
             # if there are no commas, we see if it makes sense
             # to turn, e.g. whole-wheat bread into bread, whole-wheat
@@ -443,13 +461,13 @@ class KeyManagerOldSchool:
 
 class KeyDictionary:
     def __init__ (self, rm):
-        """We create a readonly dictionary based on the metakit iview table."""
+        """We create a readonly dictionary based on the metakit ingredients_table table."""
         self.rm = rm
         self.default = defaults.keydic
 
     def has_key (self, k):
         debug('has_key testing for %s'%k,1)
-        if self.rm.fetch_one(self.rm.iview,item=k): return True
+        if self.rm.fetch_one(self.rm.ingredients_table,item=k): return True
         elif self.default.has_key(k): return True
         else: return False
 
@@ -464,19 +482,19 @@ class KeyDictionary:
 
     def __getitem__ (self, k):
         kvw = self.rm.fetch_count(
-            self.rm.iview,
+            self.rm.ingredients_table,
             'ingkey',
             sort_by=('count',-1),
             item=k
             )
 
     def keys (self):
-        ll = self.rm.get_unique_values('item',self.rm.iview,deleted=False)
+        ll = self.rm.get_unique_values('item',self.rm.ingredients_table,deleted=False)
         ll.extend(self.default.keys())
         return ll
 
     def values (self):
-        ll = self.rm.get_unique_values('ingkey',self.rm.iview,deleted=False)
+        ll = self.rm.get_unique_values('ingkey',self.rm.ingredients_table,deleted=False)
         ll.extend(self.default.values())
         return ll
 
@@ -500,6 +518,13 @@ cooking_verbs=["cored",
                "heated",
                "warmed",
                "chilled"]
+
+def get_keymanager (*args, **kwargs):
+    try:
+        return KeyManager(*args,**kwargs)
+    except KeyManager, km:
+        return km
+
 if __name__ == '__main__':
 
     def timef (f):
