@@ -10,21 +10,21 @@ class GourmetXML2Plugin (ImporterPlugin):
     mimetypes = ['text/xml','application/xml','text/plain']
 
     def test_file (self, filename):
-        Tester('.*<gourmetDoc[> ]').test(filename)
+        return Tester('.*<gourmetDoc[> ]').test(filename)
 
-    def get_importer (self, **args):
-        return gxml2_importer.converter(args['file'],args['rd'])
+    def get_importer (self, filename):
+        return gxml2_importer.Converter(filename)
 
 
 class GourmetXMLPlugin (ImporterPlugin):
 
-    name = _('Gourmet XML File')
+    name = _('Gourmet XML File (Obsolete)')
     patterns = ['*.xml','*.grmt','*.gourmet']
     mimetypes = ['text/xml','application/xml','text/plain']
 
     def test_file (self, filename):
-        importer.Tester('.*<recipeDoc[> ]').test(filename)
+        return Tester('.*<recipeDoc[> ]').test(filename)
 
-    def get_importer (self, **args):
-        return gxml_importer.converter(args['file'],args['rd'])
+    def get_importer (self, filename):
+        return gxml_importer.Converter(filename)
 
