@@ -1,5 +1,6 @@
 from gourmet.plugin import ExporterPlugin
 import html_exporter
+from gettext import gettext as _
 
 WEBPAGE = _('HTML Web Page')
 
@@ -22,7 +23,7 @@ class HtmlExporterPlugin (ExporterPlugin):
             )
 
     def do_single_export (self, args)    :
-        html_exporter.html_exporter(
+        he = html_exporter.html_exporter(
             args['rd'],
             args['rec'],
             args['out'],
@@ -30,6 +31,7 @@ class HtmlExporterPlugin (ExporterPlugin):
             mult=args['mult'],
             conv=args['conv']
             )
+        he.run()
 
     def run_extra_prefs_dialog (self):
         pass
