@@ -10,8 +10,8 @@ class MealmasterExporterPlugin (ExporterPlugin):
     sublabel = _('Exporting recipes to MealMaster file %(file)s.')
     single_completed_string = _('Recipe saved as MealMaster file %(file)s')
     filetype_desc = MMF
-    saveas_filters = [MMF,['text/mmf'],['*.mmf','*.MMF']]
-    saveas_single_filters = [MMF,['text/html'],['']]
+    saveas_filters = [MMF,['text/mmf','text/plain'],['*.mmf','*.MMF']]
+    saveas_single_filters = saveas_filters
 
     def get_multiple_exporter (self, args):
         return exporter.ExporterMultirec(
@@ -25,12 +25,13 @@ class MealmasterExporterPlugin (ExporterPlugin):
             exporter=mealmaster_exporter.mealmaster_exporter)
 
     def do_single_export (self, args)    :
-        return mealmaster_exporter.mealmaster_exporter(args['rd'],
+        e = mealmaster_exporter.mealmaster_exporter(args['rd'],
                                                        args['rec'],
                                                        args['out'],
                                                        mult=args['mult'],
                                                        change_units=args['change_units'],
                                                        conv=args['conv'])
+        e.run()
 
     def run_extra_prefs_dialog (self):
         pass
