@@ -78,7 +78,7 @@ class mmf_importer (plaintext_importer.TextImporter):
 
     committed = False
     
-    def __init__ (self,rd,filename='Data/mealmaster.mmf',
+    def __init__ (self,filename='Data/mealmaster.mmf',
                   prog=None, source=None,threaded=True,
                   two_col_minimum=38,conv=None):
         """filename is the file to parse (or filename). rd is the recData instance
@@ -98,7 +98,7 @@ class mmf_importer (plaintext_importer.TextImporter):
         self.unit_length = 2
         self.two_col_minimum = two_col_minimum
         self.last_line_was = None
-        plaintext_importer.TextImporter.__init__(self,filename,rd)#prog=prog,
+        plaintext_importer.TextImporter.__init__(self,filename)#prog=prog,
         #threaded=threaded,conv=conv)
         testtimer.end()
         
@@ -264,7 +264,7 @@ class mmf_importer (plaintext_importer.TextImporter):
         self.parse_inglist()
         if self.source:
             self.rec['source']=self.source
-        importer.importer.commit_rec(self)
+        importer.Importer.commit_rec(self)
         # blank rec
         self.committed = True
         self.in_variation=False
@@ -425,7 +425,7 @@ class mmf_importer (plaintext_importer.TextImporter):
         unit = unit.strip()
         if self.mmf.unit_conv.has_key(unit):
             unit = self.mmf.unit_conv[unit]
-        importer.importer.add_unit(self,unit)
+        importer.Importer.add_unit(self,unit)
         testtimer.end()
 
 def split_fields (strings, char=" "):
