@@ -71,7 +71,7 @@ def get_recipe_hash (recipe_object):
     return m.hexdigest()
 
 def hash_recipe (rec, rd, conv=None):
-    if not conv: conv = convert.Converter()
+    if not conv: conv = convert.get_converter()
     rechash = get_recipe_hash(rec)
     inghash = get_ingredient_hash(rd.get_ings(rec),conv)
     return rechash,inghash
@@ -108,12 +108,15 @@ def apply_line_markup (line, markup):
     if len(markup) < len(line):
         markup += ' '*(len(line)-len(markup))
     for n in range(len(line)):
-        if markup[n]==' ':
+        if markup[n]==' ' or markup[n]=='\n':
             tag = None
         elif markup[n]=='+':
             tag = 'add'
         elif markup[n]=='-':
             tag = 'del'
+        else:
+            #print "WARNING: don't recognize diff tag \"%s\""%markup[n]
+            tag = None
         if tag != current_tag:
             if current_tag:
                 out += '</%s>'%current_tag
